<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class HtmlElement extends ZENNO_ABSTRACT
{
    /**
     * @var int
     */
    public $_zp_id;
    
    /**
     * @var bool
     */
    public $_errorDetected = false;
    
    /**
     * @var string
     */
    public $_address;
    
    /**
     * @var CommandTransfer
     */
    public $_transfer;

    public $_seprator = ':';

    /**
     * 
     * @return HtmlElement
     * @param int|string $zp_id
     * @param string $address
     * @param CommandTransfer $transfer
     */
    public function __construct($zp_id, $address, CommandTransfer $transfer)
    {
        $this->_zp_id = $zp_id == '' ? -1 : $zp_id;
        $this->_address = $address;
        $this->_transfer = $transfer;
    }

    /**
     * была ли ошибка при выполнении последней команды
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }

    /**
     * пустой (не имеет представления на сервере)
     * 
     * @return bool 
     */
    protected function _getIsVoid()
    {
        return $this->_zp_id == -1;
    }

    /**
     * Id, представляющий элемент на сервере
     * 
     * @return int
     */
    protected function  _getZP_ID()
    {
        return $this->_zp_id;
    }

    /**
     * Родительская вкладка
     * 
     * @return Tab
     */
    protected function _getParentTab()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ParentTab',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTab($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Tab::GetNull();
        }
    }
    
    /**
     * Родительский документ
     * 
     * @return Document
     */
    protected function _getParentDocument()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ParentDocument',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToDocument($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Document::GetNull();
        }
    }
    
    
    /**
     * Родительский документ 
     * 
     * @return HtmlElement 
     */
    protected function _getParentElement()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ParentElement',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Родительская форма 
     * 
     * @return HtmlElement
     */
    protected function _getParentForm()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ParentForm',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Родительское тело
     * 
     * @return HtmlElement
     */
    protected function _getParentBody()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ParentBody',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Первый дочерний элемент 
     * 
     * @return HtmlElement
     */
    protected function _getFirstChild()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FirstChild',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }

    /**
     * Последний дочерний элемент
     * 
     * @return HtmlElement
     */
    protected function _getLastChild()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_LastChild',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }


    /**
     * Следующий элемент (если следующий элемент на уровень выше, то вернет null)
     * 
     * @return HtmlElement
     */
    protected function _getNextSibling()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_NextSibling',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Предыдущий элемент (если предыдущий элемент на уровень выше, то вернет null)
     * 
     * @return HtmlElement
     */
    protected function _getPreviousSibling()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_PreviousSibling',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    /// <summary> </summary>
    /**
     * Dom текст элемента
     * 
     * @return string
     */
    protected function _getOuterHtml()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_OuterHtml',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Отображаемый html текст внутри тега элемента 
     * 
     * @return string
     */
    protected function _getInnerHtml()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_InnerHtml',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }


    /**
     * Тэг элемента
     * 
     * @return string
     */
    protected function _getTagName()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_TagName',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    /**
     * Полный тэг элемента (например не input а input:radio или input:password)
     * 
     * @return string
     */
    protected function _getFullTagName()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FullTagName',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }

    /**
     * Имя элемента
     * 
     * @return string
     */
    protected function _getName()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Name',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }


    /**
     * id элемента
     * 
     * @return string
     */
    protected function _getId()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Id',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Высота элемента 
     * 
     * @return int
     */
    protected function _getHeight()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Height',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    
    
    /**
     * Ширина элемента 
     * 
     * @return int
     */
    protected function _getWidth()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Width',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    
    
    /**
     * Существует ли этот элемент 
     * 
     * @return bool
     */
    protected function _getIsNull()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_IsNull',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    
    /**
     * Смещение левого верхнего угла относительно окна таба
     * 
     * @return object - point object {x, y}
     */
    protected function _getDisplacementInTabWindow()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_DisplacementInTabWindow',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToPoint($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return (object)array('x' => 0,
                                 'y' => 0);
        }
    }
    
    
    /**
     * Смещение левого верхнего угла относительно родительского документа
     * 
     * @return object - point object {x, y}
     */
    protected function _getDisplacementInDocument()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_DisplacementInDocument',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToPoint($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return (object)array('x' => 0,
                                 'y' => 0);
        }
    }

    /**
     * отрисовать элемент в картинку
     * 
     * @return string
     * @param bool $isImage
     */
    public function DrawToBitmap($isImage)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_DrawToBitmap',
                array('zp_id', (string)$this->_zp_id,
                      'isImage', (string)$isImage), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }	
	
	/**
     * отрисовать элемент в картинку
     * 
     * @return string
	 * @param int $left
	 * @param int $top
	 * @param int $width
	 * @param int $height
     * @param bool $isImage
     */
    public function DrawPartToBitmap($left, $top, $width, $height, $isImage)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_DrawPartToBitmap',
                array('zp_id', (string)$this->_zp_id,
					  'left', (string)$left,
					  'top', (string)$top,
					  'width', (string)$width,
					  'height', (string)$height,
                      'isImage', (string)$isImage), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }

    /**
     * Выдает элемент, по содержащемуся в нем атрибуту среди детей этого элемента
     * 
     * @return HtmlElement
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     * @param int $number
     */
    public function FindChildByAttribute($tags, $attrName, $attrValue, $searchKind, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildByAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags,
                      'attrName', $attrName,
                      'attrValue', $attrValue,
                      'searchKind', $searchKind,
                      'number', (string)$number), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    /**
     *  Выдает элемент, по содержащемуся в нем атрибуту среди детей этого элемента
     *  
     * @return HtmlElementCollection
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     */
    public function FindChildrenByAttribute($tags, $attrName, $attrValue, $searchKind)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildrenByAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags,
                      'attrName', $attrName,
                      'attrValue', $attrValue,
                      'searchKind', $searchKind), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    /**
     * Выдает элемент, по id среди детей этого элемента
     * 
     * @return HtmlElement
     * @param string $id
     */
    public function FindChildById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildById',
                array('zp_id', (string)$this->_zp_id,
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по id среди детей этого элемента 
     * 
     * @return HtmlElementCollection
     * @param string $id
     */
    public function FindChildrenById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildrenById',
                array('zp_id', (string)$this->_zp_id,
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по name среди детей этого элемента 
     * 
     * @return HtmlElement
     * @param string $name
     */
    public function FindChildByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildByName',
                array('zp_id', (string)$this->_zp_id,
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по name среди детей этого элемента
     * 
     * @return HtmlElementCollection
     * @param string $name
     */
    public function FindChildrenByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildrenByName',
                array('zp_id', (string)$this->_zp_id,
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по тэгу среди детей этого элемента
     * 
     * @return HtmlElement
     * @param string $tag
     * @param int $number
     */
    public function FindChildByTag($tag, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildByTag',
                array('zp_id', (string)$this->_zp_id,
                      'tag', $tag,
                      'number', $number));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по тэгам среди детей этого элемента 
     * 
     * @return HtmlElementCollection
     * @param string $tags
     */
    public function FindChildrenByTags($tags)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_FindChildrenByTags',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдать детей
     * 
     * @return HtmlElementCollection
     * @param bool $recursive
     */
    public function GetChildren($recursive)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_GetChildren',
                array('zp_id', (string)$this->_zp_id,
                      'recursive', ($recursive ? 'true' : 'false')));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Добавить дочрний элемет
     * 
     * @return void
     * @param HtmlElement $child
     */
    public function AppendChild($child)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_AppendChild',
                array('zp_id', (string)$this->_zp_id,
                      'child', (string)$child->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Удалить дочерний элемент (станет null т.к. не будет больше принадлежать никакому документу)
     * 
     * @return void 
     * @param HtmlElement $child
     */
    public function RemoveChild($child)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_RemoveChild',
                array('zp_id', (string)$this->_zp_id,
                      'child', (string)$child->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }


    /**
     * Вставить указанный элемент перед собой
     * 
     * @return void
     * @param HtmlElement $insertingElement
     */
    public function InsertElementBefore($insertingElement)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_InsertElementBefore',
                array('zp_id', (string)$this->_zp_id,
                      'insertingElement', (string)$insertingElement->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Вставить указанный элемент после себя
     * 
     * @return void
     * @param HtmlElement $insertingElement
     */
    public function InsertElementAfter($insertingElement)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_InsertElementAfter',
                array('zp_id', (string)$this->_zp_id,
                      'insertingElement', (string)$insertingElement->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Установить текст атрибута
     * 
     * @return void
     * @param string $attrName
     * @param string $attrValue
     */
    public function SetAttribute($attrName, $attrValue)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_SetAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'attrName', $attrName,
                      'attrValue', $attrValue), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Взять текст атрибута 
     * 
     * @return string
     * @param string $attrName
     */
    public function GetAttribute($attrName)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_GetAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'attrName', $attrName));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }


    /**
     * Удалить атрибут
     * 
     * @return void
     * @param string $attrName
     */
    public function RemoveAttribute($attrName)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_RemoveAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'attrName', $attrName));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Вызвать событие 
     * 
     * @return void
     * @param string $eventName
	 * @param bool $emulation
     */
    public function RiseEvent($eventName, $emulation = true)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_RizeEvent',
                array('zp_id', (string)$this->_zp_id,
                      'eventName', $eventName, "emulation", ($emulation ? 'true' : 'false') ));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
			usleep(300000);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Заменить дочерний элемент
     * 
     * @return void
     * @param HtmlElement $newChild
     * @param HtmlElement $oldChild
     */
    public function ReplaceChild($newChild, $oldChild)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_ReplaceChild',
                array('zp_id', (string)$this->_zp_id,
                      'newChild', (string)$newChild->ZP_ID,
                      'oldChild', (string)$oldChild->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Взять значение элемента  
     * 
     * @return String
     */
    public function GetValue()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_GetValue', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Установить значение элемента 
     * 
     * @return void
     * @param string $value
     * @param bool $emulation
     */
    public function SetValue($value, $emulation)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_SetValue',
                array('zp_id', (string)$this->_zp_id,
                      'value', $value,
                      'emulation', ($emulation ? 'true' : 'false')), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Взять значение элемента 
     * 
     * @return string
     */
    public function GetSelectedItems()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_GetSelectedItems', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Взять значение элемента
     * 
     * @return void
     * @param string $value
     */
    public function SetSelectedItems($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_SetSelectedItems',
                array('zp_id', (string)$this->_zp_id,
                      'value', $value), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Перевести фокус на элемент 
     * 
     * @return void
     */
    public function Focus()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Focus', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Кликнуть по элементу 
     * 
     * @return void
     */
    public function Click()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'HE_Click', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
	
	
    
    
    /**
     * Возвращает пустой элемент 
     * 
     * @return HtmlElement 
     */
    public static function GetNull()
    {
        return new HtmlElement(-1, '', null);
    }
}