<?php
set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__));
require_once('Abstract.php');
require_once('CommandTransfer.php');
require_once('DocumentCollection.php');
require_once('Document.php');
require_once('HtmlElementCollection.php');
require_once('HtmlElement.php');
require_once('Tab.php');
require_once('ZennoPoster.php');
require_once('ZPConst.php');
require_once('ZPConverter.php');

class Instance extends ZENNO_ABSTRACT
{
    /**
     * @var bool
     */
    protected $_errorDetected = false;

    /**
     * @var string
     */
    public $_address = '';

    /**
     * @var CommandTransfer 
     */
    public $_transfer = null;

    /**
     * @var string
     */
    protected $_portUrl = '';
    
    /**
     * 
     * 
     * @param 
     * @return 
     */
    public function __construct()
    {
        $args = func_get_args();

        if (count($args) == 3) {
            $this->_transfer = new CommandTransfer($args[0], $args[1], '');
            $this->_portUrl = "{$args[1]}:$args[0]}";
            $this->_address = $args[2];
        } else if (count($args) == 2) {
            try
            {
                $parameters = explode(':', $portUrl, 2);
                if (count($parameters) == 2) {
                    $this->_transfer = new CommandTransfer($parameters[0], (int)$parameters[1], '');
                }
            }
            catch (Exception $e){
                die($e);
            }
            $this->_errorDetected = true;
            $this->_address = $args[1];
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return ($this->_transfer == null);
    }

    /**
     * 
     * @return string
     */
    protected function _getZP_ID()
    {
        return $this->_portUrl;
    }

    /**
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }

    /**
     * 
     * @return array - array of Tab object
     */
    protected function _getAllTabs()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string $message
             */
            $message = CommandTransfer::CreateMessage($this->_address, 'Instance_AllTabs', '', '');

            /**
             * @var string $result
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTabArray($result,
                $this->_address, 
                $this->_transfer,
                &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return array();
        }
    }
    
    /**
     * 
     * @return Tab 
     */
    protected function _getMainTab()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address, 'Instance_MainTab', '', '');

            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTab($result,
                $this->_address,
                $this->_transfer,
                &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Tab::GetNull();
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getLoadPictures()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                    'Instance_IsLoadPictures_Get', '', '');
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setLoadPictures($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsLoadPictures_Set',
                'value', (string)$value);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }

    /**
     * 
     * @return bool
     */
    protected function _getAllowPopUp()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsAllowPopUp_Get', '', '');
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }

    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setAllowPopUp($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsAllowPopUp_Set',
                'value',
                (string)$value);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getBackGroundSoundsPlay()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsBackGroundSoundsPlay_Get', '', '');
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }

    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setBackGroundSoundsPlay($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsBackGroundSoundsPlay_Set',
                'value', 
                (string)$value);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getDownloadActiveX()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address, 
                'Instance_IsDownloadActiveX_Get', '', '');
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setDownloadActiveX($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid)
        {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsDownloadActiveX_Set',
                'value',
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     *  
     * @return bool
     */
    protected function _getDownloadFrame()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsDownloadFrame_Get', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setDownloadFrame($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsDownloadFrame_Set',
                'value',
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getUseJavaApplets()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsUseJavaApplets_Get', '', '');
            $result = $this->_transfer.SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setUseJavaApplets($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsUseJavaApplets_Set',
                'value', 
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, $this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getRunActiveX()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsRunActiveX_Get', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }

    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setRunActiveX($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsRunActiveX_Set',
                'value',
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }

    /**
     * 
     * @return bool
     */
    protected function _getUseJavaScripts()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsUseJavaScripts_Get', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }

    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setUseJavaScripts($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsUseJavaScripts_Set',
                'value',
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return bool
     */
    protected function _getDownloadVideos()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsDownloadVideos_Get', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }

    /**
     * 
     * @return void
     * @param object $value
     */
    protected function _setDownloadVideos($value)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_IsDownloadVideos_Set',
                'value',
                (string)$value);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     */
    public function ClearCookie()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_ClearCookie', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     */
    public function ClearCache()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_ClearCache', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return Tab
     */
    public function NewTab($address)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_NewTab', 'address', 'address');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTab($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Tab::GetNull();
        }
    }
    
    /**
     * 
     * @return Tab
     * @param string $address
     */
    public function GetTabByAddress($address)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_GetTabByAddress', 'address', $address);
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTab($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Tab::GetNull();
        }
    }
    
    /**
     * 
     * @return void
     */
    public function CloseAllTabs()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_CloseAllTabs', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     */
    public function SetOkAlertPolicy($answer)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetOkAlertPolicy',
                'answer',
                (string)$answer);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     * @param string $value
     */
    public function SetOkCancelConfirmPolicy($answer, $value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetOkCancelConfirmPolicy',
                'answer',
                (string)$answer);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     * @param string $value
     */
    public function SetJSValuePromptPolicy($answer, $value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetJSValuePromptPolicy',
                array('answer', $answer, 'value', $value));
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     * @param string $value
     */
    public function SetFileUploadPolicy($answer, $value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetFileUploadPolicy',
                array('answer', $answer, 'value', $value));
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     * @param string $login
     * @param string $pass
     */
    public function SetAuthorizationPolicy($answer, $login, $pass)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetAutorisationPolicy',
                array('answer', $answer,
                      'loginPass', "{$login}:{$pass}"));
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param string $answer
     */
    public function SetCertificatesPolicy($answer)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetCertificatesPolicy',
                'answer',
                (string)$answer);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     */
    public function SetDefaultPreference()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetDefoultPreference', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
	
	/**
     *   ( : random, default)
     * 
     * @return void
     * @param string $headerName
     * @param string $value
     */
    public function SetHeader($headerName, $value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetHeader',
                array('zp_id', (string)$this->_zp_id,
                      'headerName', $headerName,
                      'value', $value), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }

    /**
     *     
     * 
     * @return void
     */
    public function HeadersToDefault()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_HeadersToDefault', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }

    
    /**
     * 
     * @return void
     * @param string $ip
     * @param int $port
     * @param bool $isSocks
     */
    public function SetProxy($ip, $port, $isSocks)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetProxy',
                array('ip', $ip,
                      'port', (string)$port,
                      'isSocks', (string)$isSocks));
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return string
     */
    public function GetProxy()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_GetProxy', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    /**
     * 
     * @return void
     */
    public function ClearProxy()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_ClearProxy', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return string
     * @param string $stepId
     * @param string $branchName
     */
    public function GetTemplateValue($stepId, $branchName)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_GetTemlateValue',
                array('stepId', $stepId,
                      'branchName', $branchName),
                null);
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    /**
     * 
     * @return void
     * @param string $stepId
     * @param string $branchName
     * @param string $value
     */
    public function SetTemplateValue($stepId, $branchName, $value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_SetTemlateValue', 
                array('stepId', $stepId,
                      'branchName', $branchName,
                      'value', $value),
                array());
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return string
     * @param string $macrossType
     * @param array $parameters - array of strings
     */
    public function RiseMacros($macrosType, $parameters)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                'Instance_RiseMacros',
                array('macrosType', $macrosType),
                $parameters);
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    /**
     * 
     * @return Instance
     */
    public static function GetNull()
    {
        return new Instance('', '');
    }
}