<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class Tab extends ZENNO_ABSTRACT
{
    /**
     * @var int
     */
    protected $_zp_id;
    
    /**
     * @var bool
     */
    protected $_errorDetected = false;
    
    /**
     * @var string
     */
    protected $_address;
    
    /**
     * @var CommandTransfer
     */ 
    protected $_transfer;

    /**
     * Адреса могут совпадать у разных табов, поэтому используем Id
     * 
     * @return 
     * @param object $zp_id
     * @param object $address
     * @param object $transfer
     */
    public function __construct($zp_id, $address, CommandTransfer $transfer)
    {
        $this->_zp_id = ($zp_id == '') ? -1 : $zp_id;
        
        $this->_address = $address;
        $this->_transfer = $transfer;
    }

    /**
     * пустой (не имеет представления на сервере)
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return $this->_zp_id == -1;
    }
    

    /**
     * была ли ошибка при выполнении последней команды 
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }
    

    /**
     * Id, представляющий вкладку на сервере 
     * 
     * @return int 
     */
    protected function _getZP_ID()
    {
        return $this->_zp_id;
    }
    ///// <summary> Адрес в инстансе </summary>
    //public String AddressZP
    //{
    //    get { return address; }
    //}
    

    /**
     * Все загруженные документы 
     * 
     * @return DocumentCollection 
     */
    protected function _getAllDocuments()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_AllDocuments',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToDocumentCollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return DocumentCollection::GetNull();
        }
    }

    /**
     * Первый документ 
     * 
     * @return Document
     */
    protected function _getMainDocument()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_MainDocument',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
			return ZPConverter::ConvertToDocument($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Document::GetNull();
        }
    }
	
    /**
     * getiing document by address 
     * 
     * @return Document
	 * @param string $docAddress
     */
    public function GetDocumentByAddress($docAddress)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_GetDocumentByAddress',
                array('zp_id', (string)$this->_zp_id,
				"docAddress", $docAddress));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToDocument($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Document::GetNull();
        }
    }

    /**
     * Первое тело
     * 
     * @return HtmlElement 
     */
    protected function _getMainBody()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_MainBody',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    

    /**
     * Url на котором находится страница этой вкладки
     * 
     * @return string
     */
    protected function _getURL()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_URL',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }

    /**
     * Domain на котором находится страница этой вкладки 
     * 
     * @return string
     */
    protected function _getDomain()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Domain',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /// <summary>  </summary>
    /**
     * Заголовок таба
     * 
     * @return string
     */
    protected function _getTitle()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Title',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    

    /**
     * Handle окна браузера 
     * 
     * @return inte
     */
    protected function _getHandle()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Handle',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    

    /**
     * Занят ли таб загрузкой страницы
     * 
     * @return bool
     */
    protected function _getIsBusy()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_IsBusy',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);		
            return ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return false;
        }
    }


    /**
     *  Существует ли этот таб
     *  
     * @return bool 
     */
    protected function _getIsNull()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_IsNull',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return false;
        }
    }


    /**
     * Таймаут зависания загрузки страницы (сек)
     * 
     * @return int
     */
    protected function _getNavigateTimeout()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_NavigateTimeout_Get',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    
    /**
     * Таймаут зависания загрузки страницы (сек)
     * 
     * @return boid
     * @param string|int $value
     */
    protected function _setNavigateTimeout($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_NavigateTimeout_Set',
                array('zp_id', (string)$this->_zp_id,
                      'value', (string)$value));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Выдает текст DOM модели страницы
     * 
     * @return string
     */
    protected function _getDomText()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_DomText',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    

    /**
     * Выдает текст страницы, который видит пользователь
     * 
     * @return string
     */
    protected function _getPageText()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_PageText',
                'zp_id',
                (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }

    /**
     * Ждем загрузки страницы 
     * 
     * @return void
     */
    public function WaitDownloading()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
		     /**
             * @var int
             */
            $timeout = $this->NavigateTimeout;
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_WaitDownloading', 'zp_id', (string)$this->_zp_id);
            $this->_transfer->SendMessageWait($message, $timeout);
        } else {
            $this->_errorDetected = true;
        }
    }


    /**
     * Останавливаем загрузку страницы 
     * 
     * @return void
     */
    public function Stop()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Stop', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Переходим на указанную страницу
     * 
     * @return void
     * @param string $url
     */
    public function Navigate($url)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Navigate', 
                array('zp_id', (string)$this->_zp_id, 
                      'url', $url));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
			usleep(300000);
            ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Выдает элемент, находящийся в указанных координатах
     * 
     * @return HtmlElement
     * @param int $x
     * @param int $y
     */
    public function GetElementFromPoint($x, $y)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_GetElementFromPoint', 
                array('zp_id', (string)$this->_zp_id, 
                      'x', $x,
                      'y', $y));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по содержащемуся в нем атрибуту
     * 
     * @return HtmlElement
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     * @param int $number
     */
    public function FindElementByAttribute($tags, $attrName, $attrValue, $searchKind, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementByAttribute', 
                array('zp_id', (string)$this->_zp_id, 
                      'tags', $tags,
                      'attrName', $attrName,
                      'attrValue', $attrValue,
                      'searchKind', $searchKind,
                      'number', (string)$number), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по содержащемуся в нем атрибуту
     * 
     * @return HtmlElementCollection
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     */
    public function FindElementsByAttribute($tags, $attrName, $attrValue, $searchKind)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementsByAttribute', 
                array('zp_id', (string)$this->_zp_id, 
                       'tags', $tags,
                       'attrName', $attrName,
                       'attrValue', $attrValue,
                       'searchKind', $searchKind), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    

    /**
     * Выдает элемент, по id
     * 
     * @return HtmlElement
     * @param string $id
     */
    public function FindElementById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementById', 
                array('zp_id', (string)$this->_zp_id, 
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по id
     * 
     * @return HtmlElementCollection
     * @param string $id
     */
    public function FindElementsById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementsById', 
                array('zp_id', (string)$this->_zp_id, 
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по name
     * 
     * @return HtmlElement
     * @param string $name
     */
    public function FindElementByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementByName', 
                array('zp_id', (string)$this->_zp_id, 
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по name
     * 
     * @return HtmlElementCollection
     * @param string $name
     */
    public function FindElementsByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementsByName', 
                array('zp_id', (string)$this->_zp_id, 
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по тэгу
     * 
     * @return HtmlElement
     * @param string $tag
     * @param int $number
     */
    public function FindElementByTag($tag, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementByTag', 
                array('zp_id', (string)$this->_zp_id, 
                      'tag', $tag,
                      'number', $number));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по тэгам (поддерживает несколько тэгов)
     * 
     * @return HtmlElementCollection
     * @param string $tags
     */
    public function FindElementsByTags($tags)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_FindElementsByTags', 
                array('zp_id', (string)$this->_zp_id, 
                      'tags', $tags));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
	
    
    /**
     * Выдает текст страницы, который видит пользователь
     * 
     * @return string
     * @param string $encoding
     */
    public function GetSourceText($encoding)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_GetSourceText', 
                array('zp_id', (string)$this->_zp_id, 
                      'encoding', $encoding));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Устанавливает текст страницы, который видит пользователь
     * 
     * @return void
     * @param string $value
     */
    public function SetSourceText($value)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_SetSourceText', 
                array('zp_id', (string)$this->_zp_id, 
                      'value', $value),
                null);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Самозакрывашка
     * 
     * @return void
     */
    public function Close()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Tab_Close', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Возвращает пустой таб
     * 
     * @return Tab
     */
    public static function GetNull()
    {
        return new Tab(-1, '', null);
    }

}