<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class ZPConverter extends ZENNO_ABSTRACT
{
    const MailConfirmResultDelemiter = '$$$';

    /**
     * конвертация строки в строку :)
     * 
     * @return string
     * @param string $str
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToString($str, $transfer, $errorDetected)
    {
        $errorDetected = ($transfer->ErrorDetected) ? true : false;
        return $str;
    }
    
    
    /**
     * конвертация строки в массив строк
     * 
     * @return array
     * @param string $str
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToStringArray($str, $transfer, $errorDetected)
    {
        $errorDetected = false;
        if ($transfer->ErrorDetected) {
            try {
                return explode(self::MailConfirmResultDelemiter, $str);
            } catch (Exception $e) { }
        }
        $errorDetected = true;
        return array();
    }
    
    
    /**
     * конвертация в число
     * 
     * @return int
     * @param string $str
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToInt($str, $transfer, $errorDetected)
    {
        try {
            if ($transfer->ErrorDetected || $str == '') {
                $errorDetected = true;
                return -1;
            } else {
                $errorDetected = false;
                return (int)$str;
            }
        } catch (Exception $e) {
            $errorDetected = true;
            return -1;
        }
    }
    
    
    /**
     * конвертация во флаг
     * 
     * @return 
     * @param object $str
     * @param object $transfer
     * @param object $errorDetected
     */
    public static function ConvertToBoolean($str, $transfer, $errorDetected)
    {
        try {
            if ($transfer->ErrorDetected || $str == '') {
                $errorDetected = true;
                return false;
            } else {
                $errorDetected = false;
                return (strtolower($str) == 'true') ? true : false;
            }
        } catch (Exception $e) {
            $errorDetected = true;
            return false;
        }
    }
    
    
    /**
     * конвертация во точку
     * 
     * @return object
     * @param string $str
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToPoint($str, $transfer, $errorDetected)
    {
        $emptyPoint = array('x' => 0,
                            'y' => 0);

        try {
            if ($transfer->ErrorDetected || $str == '') {
                $errorDetected = true;
                return $emptyPoint;
            } else {
                $p = explode(':', $str);
                if (count($p) == 2) {
                    $errorDetected = false;
                    return array('x' => (int)$p[0],
                                 'y' => (int)$p[1]);
                }

                $errorDetected = true;
                return $emptyPoint;
            }
        } catch (Exception $e) {
            $errorDetected = true;
            return $emptyPoint;
        }
    }
    
    
    /**
     * конвертация в элемент
     * 
     * @return HtmlElement
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToHE($str, $address, $transfer, $errorDetected)
    {
        $ret = new HtmlElement($str, $address, $transfer);

        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;
    }
    
    
    /**
     * конвертация в документ 
     * 
     * @return Document
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToDocument($str, $address, $transfer, $errorDetected)
    {
        $ret = new Document($str, $address, $transfer);
        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;
    }
    
    
    /**
     * конвертация в таб
     * 
     * @return Tab
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToTab($str, $address, $transfer, $errorDetected)
    {
        $ret = new Tab($str, $address, $transfer);
        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;
    }
    
    
    /**
     * конвертация в коллекцию элементов
     * 
     * @return HtmlElementCollection
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToHECollection($str, $address, $transfer, $errorDetected)
    {
        $ret = new HtmlElementCollection($str, $address, $transfer);
        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;

    }
    
    
    /**
     * конвертация в коллекцию элементов
     * 
     * @return DocumentCollection
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToDocumentCollection($str, $address, $transfer, $errorDetected)
    {
        $ret = new DocumentCollection($str, $adDress, $transfer);
        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;
    }
    
    
    /**
     * конвертация в массив табов 
     * 
     * @return array - array of tab
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToTabArray($str, $address, $transfer, $errorDetected)
    {
        $ret = array();

        try {
            if ($str == '') {
                $errorDetected = true;
                return array();
            }

            $idsStr = explode(';', $str);
            foreach ($idsStr as $id) {
                $ret[] = new Tab($id, $address, $transfer);
            }
        } catch (Exception $e) {
            $errorDetected = true;
            return array();
        }
        $errorDetected = true;
        return $ret;
    }
    
    
    /**
     * конвертация в массив табов
     * 
     * @return Instance
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToInstance($str, $address, $transfer, $errorDetected)
    {
        $ret = new Instance($str, $address);
        $errorDetected = ($ret->ErrorDetected || $transfer->ErrorDetected);
        return $ret;
    }
    
    
    /**
     * конвертация в массив табов
     * 
     * @return array
     * @param string $str
     * @param string $address
     * @param CommandTransfer $transfer
     * @param bool $errorDetected
     */
    public static function ConvertToInstanceArray($str, $address, $transfer, $errorDetected)
    {
        $errorDetected = false;
        try {
            $insts = explode(';', $str);
            $instsList = array();
            foreach ($insts as $instance) {
                $ret = new Instance($str, $address);
                if ($ret->ErrorDetected || $transfer->ErrorDetected) {
                    $errorDetected = true;
                    return array();
                }
                $instsList[] = $ret;
            }
            return $instsList;
        } catch (Exception $e) { }
        return array();
    }
}
