<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class ZennoPoster extends ZENNO_ABSTRACT
{
    /**
     * 
     */
    const captchaRecognitionError = "errorInCaptchaRecognition";

    /**
     * TODO: add comment
     *
     * @var int
     */
    protected $_port;

    /**
     * TODO: add comment
     * 
     * @var bool
     */
    protected $_errorDetected;

    /**
     * TODO: add comment
     * 
     * @var string
     */
    protected $_address;

    /**
     * TODO: add comment
     * 
     * @var CommandTransfer 
     */
    protected $_transfer = null;

    /**
     * TODO: add comments
     * 
     * @param string|int (address|port)
     * @param int|CommandTransfer (port|CommandTransfer object)
     * @param CommandTransfer|null (CommandTransfer object)
     * @return ZennoPoster
     */
    public function __construct()
    {
        // check type of arguments
        $args = func_get_args();

        $validArgs = true;

        if ((count($args) != 2 && count($args) != 3) ||
           // Int32 port, CommandTransfer transfer
           (count($args) == 2 &&
            (! is_int($args[0]) || !is_a($args[1], 'CommandTransfer'))) ||
           // String address, Int32 port, CommandTransfer transfer
           (count($args) == 3 &&
            (! is_string($args[0]) || ! is_int($args[1]) || !is_a($args[2], 'CommandTransfer') ))) {
                $validArgs = false;
        }

        if (! $validArgs) {
            throw new Exception('Failed arguments.');
        }


        $this->_address  = (count($args) == 2) ? 'ZP' : $args[0];
        $this->_prot     = (count($args) == 2) ? $args[0] : $args[1];
        $this->_transfer = (count($args) == 2) ? $args[1] : $args[2];
    }

    /**
     * TODO: add comment
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return ($this->_transfer == null);
    }

    protected function _getInstanceTreadsCount()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                        'ZP_InstanceTreadsCount', '', '');
            $result  = $this->_transfer->SendMessageWait(message);
            return ZPConverter::ConvertToInt($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
        return -1;
    }

    protected function _getMailConfirmTreadsCount()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address,
                        'ZP_MailConfirmTreadsCount', '', '');
            $result  = $this->_transfer->SendMessageWait(message);
            return ZPConverter::ConvertToInt($result, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->errorDetected = true;
        }
        return -1;
    }

	 /**
     * 
     * 
     * @return 
     * @param string $dllName
     * @param HtmlElement $captcha
     * @param string $parameters
     */
    public static function CaptchaRecognition($dllName, $captcha, $parameters)
    {
        if (! $capcha->isVoid) {
            $transfer = $captcha->_transfer;
            if ($transfer != null) {
                if (is_a($captcha, 'HtmlElement')) {
                    if (is_a($captcha, 'HtmlElement')) {    // HtmlElement object
                        $captchaRecognition = 'ZP_CaptchaRecognition';
                        $extraArguments = array('dllName', $dllName,
                                                'id', (string)$captcha->ZP_ID,
                                                'parameters', $parameters);
					}
                } else {
					echo "exception";
                    //throw new Exception('Argument #2 is not HtmlElement');
                }
				
                $message = CommandTransfer::CreateMessage($captcha->_address,
                                                          $captchaRecognition,
                                                          $extraArguments);
                $result = $transfer->SendMessageWait($message, 5 * 60);
                /**
                 * @var $errorDetected - local
                 */
                $temp = ZPConverter::ConvertToString($result, $transfer, &$errorDetected);
                if (! $errorDetected)
                    return $temp;
            }
        }
        return self::captchaRecognitionError;
    }
	
	
		 /**
     * 
     * 
     * @return 
	 * @param Instance instance
     * @param string $dllName
     * @param array $captchas
     * @param string $parameters
     */
    public static function CaptchaRecognitionMulti($instance, $dllName, $captchas64Str, $parameters)
    {
		if (! $instance->isVoid) {
			$transfer = $instance->_transfer;
            if ($transfer != null) {
				$captchaRecognition = 'ZP_CaptchaRecognition_Multi';
				$extraArguments = array('dllName', $dllName,
										'parameters', $parameters);
                $message = CommandTransfer::CreateMessage($captcha->_address,
                                                          $captchaRecognition,
                                                          $extraArguments,
														  $captchas64Str);
                $result = $transfer->SendMessageWait($message, 5 * 60);
                /**
                 * @var $errorDetected - local
                 */
                $temp = ZPConverter::ConvertToString($result, $transfer, &$errorDetected);
                if (! $errorDetected)
                    return $temp;
            }
		}
        return self::captchaRecognitionError;
    }

    /**
     * 
     * @return Instance|array 
     * @param mixed $count[optional]
     */
    public function CreateNewInstance($count = null)
    {
        $this->_errorDetected = false;
        var_dump($this, $this->isVoid);die;
        if (!$this->isVoid) {
            if ($count == null) {
                $newInstance = 'ZP_CreateNewInstance';
                $argName = '';
                $argValue = '';
            } else {
                $newInstance = 'ZP_CreateNewInstance_Multi';
                $argName = 'count';
                $argValue = "{$count}";
            }
            $message = CommandTransfer::CreateMessage($this->_address, $newInstance, $argName, $argValue);
            $result  = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInstance($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
        return ($count == null) ? Instance::GetNull() : array();
    }
    
    
    
    /**
     * 
     * @return array
     */
    public function GetAllInstances()
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address, 'ZP_GetAllInstances', '', '');
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInstanceArray($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
        return array();
    }
    
    /**
     * 
     * @return void
     * @param array $instances 
     */
    public function CloseInstances($instances)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $ids = array();
            foreach ($instances as $instance) {
                $ids[] = $instance->ZP_ID;
            }
            $message = CommandTransfer::CreateMessage($this->_address, 'ZP_CloseInstances', 'ids', implode(';', $ids));
            $result  = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToInstanceArray($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return void
     * @param Instance $instance
     */
    public function CloseInstance($instance)
    {
        $this->_errorDetected = false;
        if (!$this->isVoid) {
            $message = CommandTransfer::CreateMessage($this->_address, 'ZP_CloseInstance', 'id', $instance->ZP_ID);
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToInstanceArray($result, $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    /**
     * 
     * @return string
     * @param Instance $instance
     * @param string $filterName
     */
    public static function GetProxy(Instance $instance, $filterName)
    {
        if (! $instance->IsVoid) {
            $transfer = $instance->_transfer;
            if ($transfer != null) {
                $message = CommandTransfer::CreateMessage($instance->_address, 'ZP_GetProxy', 'stepId', $filterName);
                $result = $transfer->SendMessageWait($message, 25);
                /**
                 * @var $errorDetected - local
                 */
                $temp = ZPConverter::ConvertToString($result, $transfer, &$errorDetected);
                if (!$_errorDetected) {
                    return $temp;
                }
            }
        }
        return '';
    }
    
    
    /**
     * 
     * @return string|array
     * @param Instance $instance
     * @param int $timeoutSec
     * @param string $login
     * @param string $pass
     * @param string $server
     * @param int $port
     * @param bool $useSSL
     * @param bool $useHTML
     * @param string $identityRegExp
     * @param string|array $parserRegExps
     */
    public static function MailConfirm(Instance $instance, $timeoutSec,
                                $login, $pass, $server, $port, $useSSL, $useHTML, $identityRegExp, $parserRegExps)
    {
        if (!$instance->IsVoid) {
            $transfer = $instance->_transfer;

            if ($transfer != null) {
                $parameters = array('id'             , $instance->ZP_ID,
                                    'timeoutSec'     , "{$timeoutSec}", 
                                    'login'          , $login,
                                    'pass'           , $pass,
                                    'server'         , $server,
                                    'port'           , "{$port}",
                                    'useSSL'         , "{$useSSL}",
                                    'useHTML'        , "{$useHTML}",
                                    'identityRegExp' , $identityRegExp);
                if (is_array($parserRegExps)) {
                    $name = 'ZP_MailConfirm_Multi';
                    $message = CommandTransfer::CreateMessage($instance->_address, $name, $parameters, $parserRegExps);
                } else {
                    $name = 'ZP_MailConfirm';
                    $parameters['parserRegExp'] = $parserRegExps;
                    $message = CommandTransfer::CreateMessage($instance->_address, $name, $parameters);
                }
                $result = $transfer->SendMessageWait($message, 5 * 1000, $timeoutSec);
                /**
                 * @var $errorDetected - local
                 */
                $temp = ZPConverter::ConvertToStringArray($result, $transfer, &$errorDetected);
                if (!$errorDetected) {
                    return temp;
                }
            }
        }
        return is_array($parserRegExps) ? array() : '';
    }
}
